this.createjs = this.createjs || {};

(function() {
  "use strict";

  /**
   * A loader instance for loading files played via NativeAudio
   */
  function Loader(loadItem) {
    this.AbstractLoader_constructor(loadItem, true, createjs.AbstractLoader.SOUND);
  }

  // Make sure that the class uses `AbstractLoader` as a plugin
  var p = createjs.extend(Loader, createjs.AbstractLoader);

  /**
   * The main load method for the class. Essentially asks NativeAudio to perform the load, then hooks into its
   * callbacks to proxy to `_sendComplete` and `sendError`, the callbacks that the loader base class exposes to child
   * classes
   */
  p.load = function() {
    // callback for success, note the `_result` and `_rawResult` classes, which are fields that the "complete" event
    // listener is expecting to see already set on the loader instance. This can be seen in the `_markComplete` method
    // in springroll's Sound module
    var onSuccess = function(msg) {
      this._rawResult = this._result = {
        id: this._item.id,
        src: this._item.src,
        msg: msg
      };

      this._sendComplete();
    }.bind(this);

    // the callback for errors. Simply forwards the error message on through
    var onError = function(msg) {
      this._sendError(msg);
    }.bind(this);

    // Attempt to load the audio, passing in alias/id for the sound, the path to the resource, the volume, the number
    // of "voices" to use (to allow concurrent sounds)
    window.plugins.NativeAudio.preloadComplex(this._item.id, this._item.src, 1, 2, 0, onSuccess, onError);
  };

  // Make sure to attach "super class" events onto the loader via `AbstractLoader_methodName`
  createjs.CordovaNativeAudioLoader = createjs.promote(Loader, 'AbstractLoader');
})();
this.createjs = this.createjs || {};

(function() {
  "use strict";

  /**
   * Represents a SoundJS sound instance that uses native audio
   *
   * @param {String} src The src of the audio file
   * @param {Number} startTime The start time (in millis in the future to start the sound)
   * @param {Number} duration IGNORED, since cordova-native-audio does not expose the duration of the sound
   * @param {Object} playbackResource The playback resource describing the sound file. Most importantly, the id field
   *                                  provides the key that cordova-native-audio will use to play and stop the sound,
   *                                  as can be seen in the `_handleSoundReady` and `_handleStop` methods
   */
  function CordovaNativeAudioSoundInstance(src, startTime, duration, playbackResource) {
    this.AbstractSoundInstance_constructor(src, startTime, duration, playbackResource);
    this._duration = 100; // make sure this is non-zero
  }

  // Make sure that this sound instance extends a base class
  var p = createjs.extend(CordovaNativeAudioSoundInstance, createjs.AbstractSoundInstance);

  /**
   * Handler for when sound is ready to play. This is the "internal" play method for SoundJS, as can be seen
   * [here](https://github.com/CreateJS/SoundJS/blob/master/src/soundjs/AbstractSoundInstance.js#L735)
   */
  p._handleSoundReady = function() {
    var method = 'play';
    if(this.loop)  {
      method = 'loop';
    }

    window.plugins.NativeAudio.setVolumeForComplexAsset(this._playbackResource.id, this.volume);
    window.plugins.NativeAudio[method](this._playbackResource.id);
  };

  /**
   * Handler for when sound is ready to be stopped. This is the "internal" stop  method for SoundJS, as can be seen
   * [here](https://github.com/CreateJS/SoundJS/blob/master/src/soundjs/AbstractSoundInstance.js#L373)
   */
  p._handleStop = function() {
    window.plugins.NativeAudio.stop(this._playbackResource.id);
  };

  // Lastly, "promote" the plugin, so that AbstractSoundInstance methods are available on instances, via
  // `AbstractSoundInstance_methodName` (as can be seen in the constructor)
  createjs.CordovaNativeAudioSoundInstance = createjs.promote(CordovaNativeAudioSoundInstance, 'AbstractSoundInstance');
})();
this.createjs = this.createjs || {};
(function() {
  "use strict";

  /**
   * A cordova plugin for using cordova-native-audio to play sound, rather than the built-in supported browser plugins
   */
  function CordovaNativeAudioPlugin() {
    this.AbstractPlugin_constructor();

    this._capabilities = CordovaNativeAudioPlugin._capabilities;
    this._soundInstanceClass = createjs.CordovaNativeAudioSoundInstance;
    this._loaderClass = createjs.CordovaNativeAudioLoader;
  }

  // Make sure that the plugin extends the base `AbstractPlugin` class
  var p = createjs.extend(CordovaNativeAudioPlugin, createjs.AbstractPlugin);

  /**
   * The capabilities of this plugin. The supported file type is dependent on the OS playing the audio, so we use
   * Cordova's platform detection to determine which audio type is supported
   *
   * @var _capabilities
   * @static
   */
  CordovaNativeAudioPlugin._capabilities = {
    tracks: -1,
    panning: false,
    volume: true,
    mp3: window.cordova && window.cordova.platformId == 'ios',
    ogg: window.cordova && window.cordova.platformId == 'android',
  };

  /**
   * Determines if the plugin is supported. Basically, the NativeAudio plugin must be loaded on the page in order for
   * this plugin to be used. This makes fallback work properly in the browser, where there are no Cordova plugins
   * loaded on the page
   *
   * @static
   */
  CordovaNativeAudioPlugin.isSupported = function() {
    return Boolean(window.plugins && window.plugins.NativeAudio);
  };

  /**
   * A simple toString method for displaying the class
   *
   * @return {String}
   */
  p.toString = function() {
    return '[CordovaNativeAudioPlugin]';
  };

  // Make sure to give the plugin access to it's "super methods" via `AbstractPlugin_methodName`
  createjs.CordovaNativeAudioPlugin = createjs.promote(CordovaNativeAudioPlugin, "AbstractPlugin");
})();
